@@ -1,4 +1,4 @@
-# Copyright (C) 2008, Sebastian Riedel.
+# Copyright (C) 2008-2009, Sebastian Riedel.
 
 package Mojo::Script::Daemon;
 
@@ -9,7 +9,7 @@ use base 'Mojo::Script';
 
 use Mojo::Server::Daemon;
 
-__PACKAGE__->attr('description', chained => 1, default => <<'EOF');
+__PACKAGE__->attr(description => (default => <<'EOF'));
 * Start the daemon. *
 Takes a port as option, by default 3000 will be used.
     daemon
@@ -41,12 +41,13 @@ Mojo::Script::Daemon - Daemon Script
 =head1 SYNOPSIS
 
     use Mojo::Script::Daemon;
+
     my $daemon = Mojo::Script::Daemon->new;
     $daemon->run(@ARGV);
 
 =head1 DESCRIPTION
 
-L<Mojo::Script::Daemon> is a simple script interface to
+L<Mojo::Script::Daemon> is a script interface to
 L<Mojo::Server::Daemon>.
 
 =head1 ATTRIBUTES
@@ -68,4 +69,4 @@ implements the following new ones.
 
     $daemon = $daemon->run(@ARGV);
 
-=cut
\ No newline at end of file
+=cut
