@@ -11,15 +11,20 @@ __PACKAGE__->attr(
     types => (
         default => sub {
             return {
-                css   => 'text/css',
-                ico   => 'image/x-icon',
-                gif   => 'image/gif',
-                html  => 'text/html',
-                jpg   => 'image/jpeg',
-                js    => 'application/x-javascript',
-                phtml => 'text/html',
-                png   => 'image/png',
-                txt   => 'text/plain'
+                css  => 'text/css',
+                gif  => 'image/gif',
+                gz   => 'application/gzip',
+                htm  => 'text/html',
+                html => 'text/html',
+                ico  => 'image/x-icon',
+                jpeg => 'image/jpeg',
+                jpg  => 'image/jpeg',
+                js   => 'application/x-javascript',
+                png  => 'image/png',
+                tar  => 'application/x-tar',
+                txt  => 'text/plain',
+                xml  => 'text/xml',
+                zip  => 'application/zip'
             };
         }
     )
