@@ -97,6 +97,9 @@ sub dispatch {
     elsif ($e) { $self->static->serve_404($c) }
 }
 
+# This will run for every request after dispatch
+sub finalize { }
+
 # Bite my shiny metal ass!
 sub handler {
     my ($self, $tx) = @_;
@@ -104,8 +107,17 @@ sub handler {
     # Start timer
     my $start = [Time::HiRes::gettimeofday()];
 
-    # Build context and dispatch
-    $self->dispatch($self->build_ctx($tx));
+    # Context
+    my $c = $self->build_ctx($tx);
+
+    # Prepare
+    $self->prepare($c);
+
+    # Dispatch unless we already have a response code
+    $self->dispatch($c) unless $c->res->code;
+
+    # Finalize
+    $self->finalize($c);
 
     # End timer
     my $elapsed = sprintf '%f',
@@ -116,6 +128,9 @@ sub handler {
     return $tx;
 }
 
+# This will run for every request before dispatch
+sub prepare { }
+
 # This will run once at startup
 sub startup { }
 
@@ -206,10 +221,18 @@ For example in production mode, C<production_mode> will be called.
 
     $mojo->dispatch($c);
 
+=head2 C<finalize>
+
+    $mojo->finalize($c);
+
 =head2 C<handler>
 
     $tx = $mojo->handler($tx);
 
+=head2 C<prepare>
+
+    $mojo->prepare($c);
+
 =head2 C<startup>
 
     $mojo->startup($tx);
